@@ -1,14 +1,25 @@
 /* Copyright (c) 2007-2008 Timothy Wall, All Rights Reserved
- * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.  
+ *
+ * The contents of this file is dual-licensed under 2
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and
+ * Apache License 2.0. (starting with JNA version 4.0.0).
+ *
+ * You can freely decide which license you want to apply to
+ * the project.
+ *
+ * You may obtain a copy of the LGPL License at:
+ *
+ * http://www.gnu.org/licenses/licenses.html
+ *
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ *
+ * You may obtain a copy of the Apache License at:
+ *
+ * http://www.apache.org/licenses/
+ *
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
  */
 package com.sun.jna.platform;
 
@@ -79,14 +90,14 @@ public class WindowUtilsTest extends TestCase {
             }
         }
     };
-    
+
     private Robot robot;
-    
+
     protected void setUp() throws Exception {
         if (!GraphicsEnvironment.isHeadless())
             robot = new Robot();
     }
-    
+
     protected void tearDown() {
         robot = null;
         if (!GraphicsEnvironment.isHeadless()) {
@@ -96,7 +107,7 @@ public class WindowUtilsTest extends TestCase {
             }
         }
     }
-    
+
     private static final int X = 100;
     private static final int Y = 100;
     private static final int W = 100;
@@ -104,7 +115,7 @@ public class WindowUtilsTest extends TestCase {
 
     /**
      * Verfies that the specified pixel within the image has the expected color component values.
-     * 
+     *
      * @param img The image to be checked.
      * @param x The X coordinate of the pixel to be checked.
      * @param y The Y coordinate of the pixel to be checked.
@@ -117,15 +128,15 @@ public class WindowUtilsTest extends TestCase {
         int r = (rgb >> 16) & 0xFF;
         int g = (rgb >> 8) & 0xFF;
         int b = (rgb & 0xFF);
-	
+
         assertEquals(expectedRed, r);
         assertEquals(expectedGreen, g);
         assertEquals(expectedBlue, b);
     }
-    
+
     /**
      * Extracts the values of the color components at the specified pixel.
-     * 
+     *
      * @param img The concerning image.
      * @param x The X coordinate of the concerning pixel.
      * @param y The Y coordinate of the concerning pixel.
@@ -136,10 +147,10 @@ public class WindowUtilsTest extends TestCase {
         int r = (rgb >> 16) & 0xFF;
         int g = (rgb >> 8) & 0xFF;
         int b = (rgb & 0xFF);
-	
+
         return new int[]{r,g,b};
     }
-    
+
     public void xtestReveal() throws Exception {
         final int SIZE = 200;
         System.setProperty("sun.java2d.noddraw", "true");
@@ -161,8 +172,9 @@ public class WindowUtilsTest extends TestCase {
         final Window f = w;
         WindowUtils.setWindowTransparent(f, true);
         content.add(new JButton("Quit") {
-			private static final long serialVersionUID = 1L;
-			{
+            private static final long serialVersionUID = 1L;
+
+            {
                 addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent e) {
                         System.exit(0);
@@ -171,7 +183,8 @@ public class WindowUtilsTest extends TestCase {
             }
         }, BorderLayout.SOUTH);
         content.add(new JComponent() {
-			private static final long serialVersionUID = 1L;
+            private static final long serialVersionUID = 1L;
+
             public Dimension getPreferredSize() {
                 return new Dimension(SIZE, SIZE);
             }
@@ -212,7 +225,7 @@ public class WindowUtilsTest extends TestCase {
         }
     }
 
-    // Expect failure on windows and x11, since transparent pixels are not 
+    // Expect failure on windows and x11, since transparent pixels are not
     // properly captured by java.awt.Robot
     public void xtestWindowTransparency() throws Exception {
         if (GraphicsEnvironment.isHeadless())
@@ -227,8 +240,9 @@ public class WindowUtilsTest extends TestCase {
         transparent.setLocation(X, Y);
         ((JComponent)transparent.getContentPane()).setOpaque(false);
         transparent.getContentPane().add(new JComponent() {
-			private static final long serialVersionUID = 1L;
-			public Dimension getPreferredSize() {
+            private static final long serialVersionUID = 1L;
+
+            public Dimension getPreferredSize() {
                 return new Dimension(W, H);
             }
             protected void paintComponent(Graphics g) {
@@ -241,29 +255,31 @@ public class WindowUtilsTest extends TestCase {
         });
         transparent.addMouseListener(handler);
         transparent.addMouseMotionListener(handler);
-        
-        SwingUtilities.invokeAndWait(new Runnable() { public void run() {
-            background.pack();
-            background.setSize(new Dimension(W, H));
-            background.setVisible(true);
-            transparent.pack();
-            transparent.setSize(new Dimension(W, H));
-            transparent.setVisible(true);
-            transparent.toFront();
-        }});
-        
+
+        SwingUtilities.invokeAndWait(new Runnable() {
+            public void run() {
+                background.pack();
+                background.setSize(new Dimension(W, H));
+                background.setVisible(true);
+                transparent.pack();
+                transparent.setSize(new Dimension(W, H));
+                transparent.setVisible(true);
+                transparent.toFront();
+            }
+        });
+
         WindowUtils.setWindowTransparent(transparent, true);
-        
+
         //robot.delay(60000);
 
         Color sample = robot.getPixelColor(X + W/2, Y + H/2);
         assertEquals("Painted pixel should be opaque", Color.red, sample);
-        
+
         sample = robot.getPixelColor(X + 10, Y + 10);
         assertEquals("Unpainted pixel should be transparent", Color.white, sample);
     }
-    
-    // Expect failure on windows and x11, since transparent pixels are not 
+
+    // Expect failure on windows and x11, since transparent pixels are not
     // properly captured by java.awt.Robot
     public void xtestWindowAlpha() throws Exception {
         if (GraphicsEnvironment.isHeadless())
@@ -278,49 +294,55 @@ public class WindowUtilsTest extends TestCase {
         transparent.setBackground(Color.black);
         transparent.setLocation(X, Y);
         WindowUtils.setWindowAlpha(transparent, .5f);
-        
+
         transparent.addMouseListener(handler);
         transparent.addMouseMotionListener(handler);
 
-        SwingUtilities.invokeAndWait(new Runnable() { public void run() {
-            background.pack();
-            background.setSize(new Dimension(W, H));
-            background.setVisible(true);
-            transparent.pack();
-            transparent.setSize(new Dimension(W, H));
-            transparent.setVisible(true);
-        }});
-        
+        SwingUtilities.invokeAndWait(new Runnable() {
+            public void run() {
+                background.pack();
+                background.setSize(new Dimension(W, H));
+                background.setVisible(true);
+                transparent.pack();
+                transparent.setSize(new Dimension(W, H));
+                transparent.setVisible(true);
+            }
+        });
+
         //robot.delay(60000);
 
-        Point where = new Point(transparent.getX() + W/2, 
+        Point where = new Point(transparent.getX() + W/2,
                                 transparent.getY() + H/2);
         Color sample = robot.getPixelColor(where.x, where.y);
         // NOTE: w32 won't sample non-opaque windows
         if (System.getProperty("os.name").startsWith("Windows")) {
             assertFalse("Sample not transparent (w32)",
-                        sample.equals(transparent.getBackground()));
+                    sample.equals(transparent.getBackground()));
         }
         else {
             assertEquals("Sample should be 50% fg/bg",
-                         new Color(128, 128, 128), sample);
+                    new Color(128, 128, 128), sample);
         }
-        
-        SwingUtilities.invokeAndWait(new Runnable() {public void run() {
-            WindowUtils.setWindowAlpha(transparent, 1f);
-        }});
+
+        SwingUtilities.invokeAndWait(new Runnable() {
+            public void run() {
+                WindowUtils.setWindowAlpha(transparent, 1f);
+            }
+        });
         sample = robot.getPixelColor(where.x, where.y);
         assertEquals("Window should be opaque with alpha=1f",
                      transparent.getBackground(), sample);
-        
-        SwingUtilities.invokeAndWait(new Runnable() {public void run() {
-            WindowUtils.setWindowAlpha(transparent, 0f);
-        }});
+
+        SwingUtilities.invokeAndWait(new Runnable() {
+            public void run() {
+                WindowUtils.setWindowAlpha(transparent, 0f);
+            }
+        });
         sample = robot.getPixelColor(where.x, where.y);
         assertEquals("Window should be transparent with alpha=0f",
                      transparent.getBackground(), sample);
     }
-    
+
     /*
     public void testWindowRegion() throws Exception {
         if (GraphicsEnvironment.isHeadless())
@@ -339,7 +361,7 @@ public class WindowUtilsTest extends TestCase {
         Area mask = new Area(new Rectangle(0, 0, W, H));
         mask.subtract(new Area(new Rectangle(W/4, H/4, W/2, H/2)));
         WindowUtils.setWindowMask(front, mask);
-        
+
         front.addMouseListener(handler);
         front.addMouseMotionListener(handler);
 
@@ -351,14 +373,14 @@ public class WindowUtilsTest extends TestCase {
             front.setSize(new Dimension(W, H));
             front.setVisible(true);
         }});
-        
+
         Point where = front.getLocationOnScreen();
         where.translate(W/8, H/8);
         Color sample = robot.getPixelColor(where.x, where.y);
         long start = System.currentTimeMillis();
         while (!sample.equals(FOREGROUND)) {
             SwingUtilities.invokeAndWait(new Runnable() { public void run() {
-                front.toFront(); 
+                front.toFront();
             }});
             Thread.sleep(10);
             if (System.currentTimeMillis() - start > 5000)
@@ -374,61 +396,64 @@ public class WindowUtilsTest extends TestCase {
         start = System.currentTimeMillis();
         while (!sample.equals(BACKGROUND)) {
             Thread.sleep(10);
-            if (System.currentTimeMillis() - start > 1000) 
+            if (System.currentTimeMillis() - start > 1000)
                 assertEquals("Background window should show through (center) "
                              + where, BACKGROUND, sample);
             sample = robot.getPixelColor(where.x, where.y);
         }
     }
     */
-    
-    public void testDisposeHeavyweightForcer() throws Exception {
-		if (GraphicsEnvironment.isHeadless())
-			return;
-		// Forcer not required on OSX
-		if (Platform.isMac())
-			return;
 
-		Frame root = JOptionPane.getRootFrame();
-		final JWindow w = new JWindow(root);
-		w.getContentPane().add(new JLabel(getName()));
-		final Rectangle mask = new Rectangle(0, 0, 10, 10);
-		SwingUtilities.invokeAndWait(new Runnable() {
-			public void run() {
-				w.pack();
-				WindowUtils.setWindowMask(w, mask);
-				w.setVisible(true);
-			}
-		});
-		try {
-			Window[] owned = w.getOwnedWindows();
-			WeakReference<Window> ref = null;
-			for (int i = 0; i < owned.length; i++) {
-				if (owned[i].getClass().getName().indexOf("Heavy") != -1) {
-					ref = new WeakReference<Window>(owned[i]);
-					break;
-				}
-			}
-			owned = null;
-			assertNotNull("Forcer not found", ref);
-			SwingUtilities.invokeAndWait(new Runnable() {
-				public void run() {
-					WindowUtils.setWindowMask(w, WindowUtils.MASK_NONE);
-				}
-			});
-			System.gc();
-			long start = System.currentTimeMillis();
-			while (ref.get() != null) {
-				Thread.sleep(10);
-				System.gc();
-				if (System.currentTimeMillis() - start > 5000)
-					fail("Timed out waiting for forcer to be GC'd");
-			}
-			assertNull("Forcer not GC'd", ref.get());
-		} finally {
-			w.dispose();
-		}
-	}
+    public void testDisposeHeavyweightForcer() throws Exception {
+        if (GraphicsEnvironment.isHeadless()) {
+            return;
+        }
+        // Forcer not required on OSX
+        if (Platform.isMac()) {
+            return;
+        }
+
+        Frame root = JOptionPane.getRootFrame();
+        final JWindow w = new JWindow(root);
+        w.getContentPane().add(new JLabel(getName()));
+        final Rectangle mask = new Rectangle(0, 0, 10, 10);
+        SwingUtilities.invokeAndWait(new Runnable() {
+            public void run() {
+                w.pack();
+                WindowUtils.setWindowMask(w, mask);
+                w.setVisible(true);
+            }
+        });
+        try {
+            Window[] owned = w.getOwnedWindows();
+            WeakReference<Window> ref = null;
+            for (int i = 0; i < owned.length; i++) {
+                if (owned[i].getClass().getName().indexOf("Heavy") != -1) {
+                    ref = new WeakReference<Window>(owned[i]);
+                    break;
+                }
+            }
+            owned = null;
+            assertNotNull("Forcer not found", ref);
+            SwingUtilities.invokeAndWait(new Runnable() {
+                public void run() {
+                    WindowUtils.setWindowMask(w, WindowUtils.MASK_NONE);
+                }
+            });
+            System.gc();
+            long start = System.currentTimeMillis();
+            while (ref.get() != null) {
+                Thread.sleep(10);
+                System.gc();
+                if (System.currentTimeMillis() - start > 5000) {
+                    fail("Timed out waiting for forcer to be GC'd");
+                }
+            }
+            assertNull("Forcer not GC'd", ref.get());
+        } finally {
+            w.dispose();
+        }
+    }
 
     // Test for accumulation of windows with repetitive setting of mask
     public void xtestWindowDisposeBug() throws Exception {
@@ -455,7 +480,7 @@ public class WindowUtilsTest extends TestCase {
             });
         }
     }
-	
+
     public static void main(String[] args) {
         junit.textui.TestRunner.run(WindowUtilsTest.class);
     }
