@@ -29,6 +29,7 @@ import static com.sun.jna.platform.win32.WinBase.FILE_IS_ENCRYPTED;
 
 import java.io.File;
 import java.io.FileWriter;
+import java.nio.file.Files;
 import java.util.Map;
 import java.util.TreeMap;
 
@@ -605,6 +606,17 @@ public class Advapi32UtilTest extends TestCase {
         Advapi32Util.registryDeleteKey(WinReg.HKEY_CURRENT_USER, "Software", "JNA");
     }
 
+    public void testRegistryLoadAppKey() throws Exception {
+        File tempDir = Files.createTempDirectory("testRegistryLoadAppKey").toFile();
+        File registryFile = new File(tempDir, "privateregistry.bin");
+        HKEYByReference phkKey = Advapi32Util.registryLoadAppKey(registryFile.getAbsolutePath(), WinNT.KEY_ALL_ACCESS, 0);
+        Advapi32Util.registryCreateKey(phkKey.getValue(), "Test");
+        Advapi32Util.registryDeleteKey(phkKey.getValue(), "Test");
+        Advapi32Util.registryCloseKey(phkKey.getValue());
+        registryFile.delete();
+        tempDir.delete();
+    }
+
     public void testRegistryGetValues() {
         String uu = "A\\u00ea\\u00f1\\u00fcC";
         Advapi32Util.registryCreateKey(WinReg.HKEY_CURRENT_USER, "Software", "JNA");
@@ -903,6 +915,18 @@ public class Advapi32UtilTest extends TestCase {
         }
     }
 
+    /**
+     * Test TOKEN_ELEVATION structure
+     */
+    public void testIsCurrentProcessElevated() {
+        // This is either true if we're elevated or false otherwise. Just exercising the function.
+        try {
+            Advapi32Util.isCurrentProcessElevated();
+        } catch (Exception ex) {
+            fail("Encountered unknown exception - " + ex.getMessage());
+        }
+    }
+
     private File createTempFile() throws Exception{
         String filePath = System.getProperty("java.io.tmpdir") + System.nanoTime()
                 + ".text";
