@@ -423,10 +423,32 @@ public class Crypt32Test extends TestCase {
         assertTrue("The status would be true since a valid certificate chain was not passed in.", status);
     }
 
-    public void testCertEnumCertificatesInStore() {
-        String SYSTEM_STORE_NAME = "ROOT";
-        WinCrypt.HCERTSTORE hCertStore = Crypt32.INSTANCE.CertOpenSystemStore(null, SYSTEM_STORE_NAME);
+    public void testCertOpenSystemStore() {
+        WinCrypt.HCERTSTORE hCertStore = Crypt32.INSTANCE.CertOpenSystemStore(null, "ROOT");
+        enumerateRootCertificates(hCertStore);
+    }
 
+    public void testCertOpenStoreWithPointerPara() {
+        WinCrypt.HCERTSTORE hCertStore = Crypt32.INSTANCE.CertOpenStore(
+                new WinCrypt.CertStoreProviderName(WinCrypt.CERT_STORE_PROV_SYSTEM_REGISTRY_W),
+                0,
+                null,
+                WinCrypt.CERT_SYSTEM_STORE_LOCAL_MACHINE | WinCrypt.CERT_STORE_OPEN_EXISTING_FLAG | WinCrypt.CERT_STORE_READONLY_FLAG,
+                new WTypes.LPWSTR("ROOT"));
+        enumerateRootCertificates(hCertStore);
+    }
+
+    public void testCertOpenStoreWithStringPara() {
+        WinCrypt.HCERTSTORE hCertStore = Crypt32.INSTANCE.CertOpenStore(
+                new WinCrypt.CertStoreProviderName(WinCrypt.CERT_STORE_PROV_SYSTEM_REGISTRY_W),
+                0,
+                null,
+                WinCrypt.CERT_SYSTEM_STORE_LOCAL_MACHINE | WinCrypt.CERT_STORE_OPEN_EXISTING_FLAG | WinCrypt.CERT_STORE_READONLY_FLAG,
+                new WTypes.LPWSTR("ROOT").getPointer());
+        enumerateRootCertificates(hCertStore);
+    }
+
+    private void enumerateRootCertificates(HCERTSTORE hCertStore) {
         int readCertificates = 0;
         int readExtensions = 0;
 
@@ -673,8 +695,14 @@ public class Crypt32Test extends TestCase {
             crlBuilder.addExtension(Extension.cRLNumber, false, new CRLNumber(new BigInteger("2")));
             X509CRLHolder holder = crlBuilder.build(caSigner);
 
-            try (OutputStream fos = Files.newOutputStream(tempFile)) {
+            OutputStream fos = null;
+            try {
+                fos = Files.newOutputStream(tempFile);
                 fos.write(holder.getEncoded());
+            } finally {
+                if (fos != null) {
+                    fos.close();
+                }
             }
 
             return tempFile;
