@@ -159,6 +159,18 @@ public class Kernel32UtilTest extends TestCase {
         }
     }
 
+    public void testFormatMessageFromErrorCodeWithNonEnglishLocale() {
+        int errorCode = W32Errors.S_OK.intValue();
+        String formattedMsgInDefaultLocale = Kernel32Util.formatMessage(errorCode);
+        // primary and sub languages id's of the english locale, because it is present on most machines
+        String formattedMsgInEnglishLocale = Kernel32Util.formatMessage(errorCode, 9, 1);
+        if(AbstractWin32TestSupport.isEnglishLocale) {
+            assertEquals(formattedMsgInDefaultLocale, formattedMsgInEnglishLocale);
+        } else {
+            assertNotSame(formattedMsgInDefaultLocale, formattedMsgInEnglishLocale);
+        }
+    }
+
     public void testGetTempPath() {
         assertTrue(Kernel32Util.getTempPath().length() > 0);
     }
@@ -244,9 +256,18 @@ public class Kernel32UtilTest extends TestCase {
         final File tmp = File.createTempFile("testGetPrivateProfileSection", ".ini");
         tmp.deleteOnExit();
 
-        final PrintWriter writer = new PrintWriter(new BufferedWriter(new FileWriter(tmp)));
+        final PrintWriter writer0 = new PrintWriter(new BufferedWriter(new FileWriter(tmp)));
+        try {
+            writer0.println("[X]");
+        } finally {
+            writer0.close();
+        }
+
+        final String[] lines0 = Kernel32Util.getPrivateProfileSection("X", tmp.getCanonicalPath());
+        assertEquals(lines0.length, 0);
+
+        final PrintWriter writer = new PrintWriter(new BufferedWriter(new FileWriter(tmp, true)));
         try {
-            writer.println("[X]");
             writer.println("A=1");
             writer.println("foo=bar");
         } finally {
