@@ -63,6 +63,9 @@ import com.sun.jna.platform.win32.WinUser.MONITORENUMPROC;
 import com.sun.jna.platform.win32.WinUser.MONITORINFO;
 import com.sun.jna.platform.win32.WinUser.MONITORINFOEX;
 
+import javax.swing.JFrame;
+import javax.swing.JWindow;
+
 /**
  * @author dblock[at]dblock[dot]org
  */
@@ -375,6 +378,35 @@ public class User32Test extends AbstractWin32TestSupport {
         assertNull("GetAncestor result should be null", result);
     }
 
+    @Test
+    public void testGetParent() {
+        HWND desktopWindow = User32.INSTANCE.GetDesktopWindow();
+        assertNotNull("Failed to get desktop window HWND", desktopWindow);
+
+        HWND result = User32.INSTANCE.GetParent(desktopWindow);
+        assertNull("GetParent result should be null", result);
+
+        final JFrame parent = new JFrame("Parent");
+        final JWindow child = new JWindow(parent);
+
+        try {
+            parent.setVisible(true);
+            child.setVisible(true);
+
+            HWND parentHwnd = new HWND();
+            parentHwnd.setPointer(Native.getComponentPointer(parent));
+
+            HWND childHwnd = new HWND();
+            childHwnd.setPointer(Native.getComponentPointer(child));
+
+            result = User32.INSTANCE.GetParent(childHwnd);
+            assertEquals("GetParent of child should be parent", parentHwnd, result);
+        } finally {
+            child.dispose();
+            parent.dispose();
+        }
+    }
+
     @Test
     public void testGetCursorPos() {
         POINT cursorPos = new POINT();
@@ -439,6 +471,25 @@ public class User32Test extends AbstractWin32TestSupport {
         assertNull("GetActiveWindow result should be null (there is no active window)", result);
     }
 
+    @Test
+    public void testBringWindowToTop() {
+        boolean result = User32.INSTANCE.BringWindowToTop(null);
+        assertFalse("BringWindowToTop(null) result should be false", result);
+
+        final JFrame w = new JFrame("Frame to bring to top");
+        try {
+            w.setVisible(true);
+
+            HWND hwnd = new HWND();
+            hwnd.setPointer(Native.getComponentPointer(w));
+
+            result = User32.INSTANCE.BringWindowToTop(hwnd);
+            assertTrue("Couldn't bring frame to top", result);
+        } finally {
+            w.dispose();
+        }
+    }
+
     @Test
     public void testSendMessage() {
         DesktopWindow explorerProc = getWindowByProcessPath("explorer.exe");
