@@ -86,13 +86,20 @@ public class VariantTest extends TestCase {
     }
 
     public void testVariantDate() {
-        SYSTEMTIME lpSystemTime = new SYSTEMTIME();
-        Kernel32.INSTANCE.GetLocalTime(lpSystemTime);
+        SYSTEMTIME lpTestSystemTime = new SYSTEMTIME();
+        Kernel32.INSTANCE.GetLocalTime(lpTestSystemTime);
+
+        // SystemTimeToVariantTime and VariantTimeToSystemTime truncate/round off millis
+        lpTestSystemTime.wMilliseconds = 0;
 
         DoubleByReference pvtime = new DoubleByReference();
-        OleAuto.INSTANCE.SystemTimeToVariantTime(lpSystemTime, pvtime);
+        OleAuto.INSTANCE.SystemTimeToVariantTime(lpTestSystemTime, pvtime);
 
         VARIANT variantDate = new VARIANT(new DATE(pvtime.getValue()));
+
+        SYSTEMTIME lpResultSystemTime = new SYSTEMTIME();
+        OleAuto.INSTANCE.VariantTimeToSystemTime(pvtime.getValue(), lpResultSystemTime);
+        assertEquals(lpTestSystemTime.toCalendar(), lpResultSystemTime.toCalendar());
     }
 
     public void testVariantRecord() {
@@ -148,17 +155,13 @@ public class VariantTest extends TestCase {
         assertThat(new DATE(5.50d).getAsJavaDate(), equalTo(new Date(1900 - 1900, 1 - 1, 4, 12, 0, 0)));
         assertThat(new DATE(5.875d).getAsJavaDate(), equalTo(new Date(1900 - 1900, 1 - 1, 4, 21, 0, 0)));
 
-        // Test roundtripping with sub hour resolution
-        // This test allows for a rounding error of 500ms, this follows MSDN:
+        // Test roundtripping with sub-hour resolution
+        // This test requires equality, in spite of this MSDN:
         // https://msdn.microsoft.com/en-us/library/aa393691.aspx
-        // the resolution is higher, but it is not requested to be
-
-        // Date was choosen from the example that made the problem visible
-        // in testing
+        // Date was chosen from the example that made the problem visible in testing
         Date testDate = new Date(2016 - 1900, 10 - 1, 12, 2, 59, 19);
-
-        assertTrue("java.util.Date -> com.sun.jna.platform.win32.OaIdl.DATE -> java.util.Date roundtrip failed",
-                Math.abs(new DATE(testDate).getAsJavaDate().getTime() - testDate.getTime()) < 500);
+        assertEquals("java.util.Date -> com.sun.jna.platform.win32.OaIdl.DATE -> java.util.Date roundtrip failed",
+                     testDate, new DATE(testDate).getAsJavaDate());
     }
 
     public void testVariantConstructors() {
