@@ -201,6 +201,16 @@ create_callback(JNIEnv* env, jobject obj, jobject method,
       cb->conversion_flags[i] = CVT_FLOAT;
       cvt = 1;
     }
+    else if (cb->arg_types[i]->type == FFI_TYPE_UINT16 || cb->arg_types[i]->type == FFI_TYPE_SINT16) {
+      cb->java_arg_types[i+3] = &ffi_type_sint;
+      cb->conversion_flags[i] = CVT_SHORT;
+      cvt = 1;
+    }
+    else if (cb->arg_types[i]->type == FFI_TYPE_UINT8 || cb->arg_types[i]->type == FFI_TYPE_SINT8) {
+      cb->java_arg_types[i+3] = &ffi_type_sint;
+      cb->conversion_flags[i] = CVT_BYTE;
+      cvt = 1;
+    }
     else if (cb->java_arg_types[i+3]->type == FFI_TYPE_STRUCT) {
       // All callback structure arguments are passed as a jobject
       cb->java_arg_types[i+3] = &ffi_type_pointer;
@@ -278,7 +288,7 @@ create_callback(JNIEnv* env, jobject obj, jobject method,
     case 'D': cb->fptr_offset = OFFSETOF(env, CallDoubleMethod); break;
     default: cb->fptr_offset = OFFSETOF(env, CallObjectMethod); break;
     }
-    status = ffi_prep_cif_var(&cb->java_cif, java_abi, 2, argc+3, java_return_type, cb->java_arg_types);
+    status = ffi_prep_cif_var(&cb->java_cif, java_abi, 3, argc+3, java_return_type, cb->java_arg_types);
     if (!ffi_error(env, "callback setup (2)", status)) {
       ffi_prep_closure_loc(cb->closure, &cb->cif, dispatch_callback, cb,
                            cb->x_closure);
@@ -429,6 +439,14 @@ invoke_callback(JNIEnv* env, callback *cb, ffi_cif* cif, void *resp, void **cbar
 	  args[i+3] = alloca(sizeof(double));
 	  *((double *)args[i+3]) = *(float*)cbargs[i];
           break;
+        case CVT_SHORT:
+	  args[i+3] = alloca(sizeof(int));
+	  *((int *)args[i+3]) = *(short*)cbargs[i];
+          break;
+        case CVT_BYTE:
+	  args[i+3] = alloca(sizeof(int));
+          *((int *)args[i+3]) = *(char*)cbargs[i];
+          break;
         case CVT_DEFAULT:
           break;
         default:
