@@ -805,7 +805,7 @@ callCallbackWithStructByValue(TestStructureByValue (*func)(TestStructureByValue)
 
 EXPORT callback_t
 callCallbackWithCallback(cb_callback_t cb) {
-  return (*cb)((callback_t)cb);
+  return (*cb)((callback_t)(void*)cb);
 }
 
 static int32_t 
@@ -890,7 +890,8 @@ addVarArgs(const char *fmt, ...) {
     case 'l':
       sum += (int) va_arg(ap, int64_t);
       break;
-    case 'c':
+    case 's': // short (promoted to 'int' when passed through '...') 
+    case 'c': // byte/char (promoted to 'int' when passed through '...')
       sum += (int) va_arg(ap, int);
       break;
     case 'f': // float (promoted to ‘double’ when passed through ‘...’)
@@ -946,6 +947,24 @@ returnStringVarArgs2(const char *fmt, ...) {
   return cp;
 }
 
+typedef union _MixedUnion1 {
+  int intValue;
+  double doubleValue;
+} MixedUnion1;
+
+EXPORT
+int stringifyMixedUnion1(
+        char* buffer, int bufferLength,
+        int dummyInt1, double dummyDouble1,
+        MixedUnion1 union1, MixedUnion1 union2,
+        int dummyInt2, double dummyDouble2) {
+    return snprintf(
+            buffer, bufferLength - 1,
+            "dummyInt1: %d, dummyDouble1: %.0f, dummyInt2: %d, dummyDouble2: %.0f, union1.intValue: %d, union2.doubleValue: %.0f",
+            dummyInt1, dummyDouble1, dummyInt2, dummyDouble2,
+            union1.intValue, union2.doubleValue);
+}
+
 #if defined(_WIN32) && !defined(_WIN64) && !defined(_WIN32_WCE)
 ///////////////////////////////////////////////////////////////////////
 // stdcall tests
@@ -1039,6 +1058,26 @@ returnClass(JNIEnv *env, jobject arg) {
   return (*env)->GetObjectClass(env, arg);
 }
 
+typedef struct{
+  double t1[3];
+  double t2[4];
+  double t3[5];
+} DemoStructureDifferentArrayLengths;
+
+EXPORT DemoStructureDifferentArrayLengths
+returnLastElementOfComponentsDSDAL(DemoStructureDifferentArrayLengths ts, int debug) {
+  DemoStructureDifferentArrayLengths result;
+  result.t1[2] = ts.t1[2];
+  result.t2[3] = ts.t2[3];
+  result.t3[4] = ts.t3[4];
+  if(debug) {
+    printf("DemoStructureDifferentArrayLengths.t1[2]: %1.3f\n", result.t1[2]);
+    printf("DemoStructureDifferentArrayLengths.t2[3]: %1.3f\n", result.t2[3]);
+    printf("DemoStructureDifferentArrayLengths.t3[4]: %1.3f\n", result.t3[4]);
+  }
+  return result;
+}
+
 #ifdef __cplusplus
 }
 #endif
